import { AsyncDeflate, AsyncGzip, AsyncZlib, AsyncInflate, AsyncGunzip, AsyncUnzlib } from 'fflate';
const compressors = {
    'gzip': AsyncGzip,
    'deflate': AsyncZlib,
    'deflate-raw': AsyncDeflate
};
const decompressors = {
    'gzip': AsyncGunzip,
    'deflate': AsyncUnzlib,
    'deflate-raw': AsyncInflate
};
const makeMulti = (TransformStreamBase, processors, name) => {
    class BaseCompressionStream extends TransformStreamBase {
        constructor(format) {
            if (!arguments.length) {
                throw new TypeError(`Failed to construct '${name}': 1 argument required, but only 0 present.`);
            }
            const Processor = processors[format];
            if (!Processor) {
                throw new TypeError(`Failed to construct '${name}': Unsupported compression format: '${format}'`);
            }
            let compressor = new Processor();
            let cb;
            super({
                start: controller => {
                    compressor.ondata = (err, dat, final) => {
                        if (err)
                            controller.error(err);
                        else if (dat) {
                            controller.enqueue(dat);
                            if (final)
                                controller.terminate();
                        }
                        cb();
                    };
                },
                transform: chunk => new Promise(resolve => {
                    cb = resolve;
                    if (chunk instanceof ArrayBuffer)
                        chunk = new Uint8Array(chunk);
                    else if (ArrayBuffer.isView(chunk)) {
                        chunk = new Uint8Array(chunk.buffer, chunk.byteOffset, chunk.byteLength);
                    }
                    else {
                        throw new TypeError("The provided value is not of type '(ArrayBuffer or ArrayBufferView)'");
                    }
                    compressor.push(chunk);
                }),
                flush: () => new Promise(resolve => {
                    cb = resolve;
                    compressor.push(new Uint8Array(0), true);
                })
            }, {
                size: chunk => chunk.byteLength | 0,
                highWaterMark: 65536
            });
        }
    }
    return BaseCompressionStream;
};
export function makeCompressionStream(TransformStreamBase) {
    class CompressionStream extends makeMulti(TransformStreamBase, compressors, 'CompressionStream') {
    }
    return CompressionStream;
}
export function makeDecompressionStream(TransformStreamBase) {
    class DeompressionStream extends makeMulti(TransformStreamBase, decompressors, 'DecompressionStream') {
    }
    return DeompressionStream;
}
